#! /usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_process_1 = __importDefault(require("process"));
const list_files_js_1 = require("../modules/helpers/list-files.js");
const get_arg_js_1 = require("../parsers/get-arg.js");
const files_js_1 = require("../configs/files.js");
const get_runtime_js_1 = require("../parsers/get-runtime.js");
const format_js_1 = require("../services/format.js");
const kill_js_1 = require("../modules/helpers/kill.js");
const env_js_1 = require("../modules/helpers/env.js");
const map_tests_js_1 = require("../services/map-tests.js");
const watch_js_1 = require("../services/watch.js");
const poku_js_1 = require("../modules/essentials/poku.js");
const write_js_1 = require("../services/write.js");
const options_js_1 = require("../parsers/options.js");
(async () => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    const configFile = (0, get_arg_js_1.getArg)('config') || (0, get_arg_js_1.getArg)('c', '-');
    const defaultConfigs = await (0, options_js_1.getConfigs)(configFile);
    const dirs = (() => {
        var _a;
        const includeArg = (0, get_arg_js_1.getArg)('include'); // deprecated
        if (includeArg !== undefined) {
            return includeArg.split(',');
        }
        return ((_a = (0, get_arg_js_1.getPaths)('-')) !== null && _a !== void 0 ? _a : ((defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.include)
            ? Array.prototype.concat(defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.include)
            : ['.']));
    })();
    const platform = (0, get_arg_js_1.getArg)('platform');
    const filter = (_a = (0, get_arg_js_1.getArg)('filter')) !== null && _a !== void 0 ? _a : defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.filter;
    const exclude = (_b = (0, get_arg_js_1.getArg)('exclude')) !== null && _b !== void 0 ? _b : defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.exclude;
    const killPort = (0, get_arg_js_1.getArg)('kill-port');
    const killRange = (0, get_arg_js_1.getArg)('kill-range');
    const killPID = (0, get_arg_js_1.getArg)('kill-pid');
    const denoAllow = (_c = (0, get_arg_js_1.argToArray)('deno-allow')) !== null && _c !== void 0 ? _c : (_d = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.deno) === null || _d === void 0 ? void 0 : _d.allow;
    const denoDeny = (_e = (0, get_arg_js_1.argToArray)('deno-deny')) !== null && _e !== void 0 ? _e : (_f = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.deno) === null || _f === void 0 ? void 0 : _f.deny;
    const denoCJS = ((_g = (0, get_arg_js_1.getArg)('deno-cjs')) === null || _g === void 0 ? void 0 : _g.split(',').map((a) => a.trim()).filter((a) => a)) ||
        (0, get_arg_js_1.hasArg)('deno-cjs') ||
        ((_h = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.deno) === null || _h === void 0 ? void 0 : _h.cjs);
    const parallel = (0, get_arg_js_1.hasArg)('parallel') || (0, get_arg_js_1.hasArg)('p', '-') || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.parallel);
    const quiet = (0, get_arg_js_1.hasArg)('quiet') || (0, get_arg_js_1.hasArg)('q', '-') || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.quiet);
    const debug = (0, get_arg_js_1.hasArg)('debug') || (0, get_arg_js_1.hasArg)('d', '-') || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.debug);
    const failFast = (0, get_arg_js_1.hasArg)('fail-fast') || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.failFast);
    const watchMode = (0, get_arg_js_1.hasArg)('watch') || (0, get_arg_js_1.hasArg)('w', '-');
    const hasEnvFile = (0, get_arg_js_1.hasArg)('env-file');
    const concurrency = (() => {
        if (!(parallel || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.parallel))) {
            return undefined;
        }
        const value = Number((0, get_arg_js_1.getArg)('concurrency'));
        return Number.isNaN(value) ? defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.concurrency : value;
    })();
    const tasks = [];
    if (killPort || ((_j = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _j === void 0 ? void 0 : _j.port)) {
        const ports = (killPort === null || killPort === void 0 ? void 0 : killPort.split(',').map(Number)) || ((_k = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _k === void 0 ? void 0 : _k.port) || [];
        tasks.push(kill_js_1.kill.port(ports));
    }
    if (killRange || ((_l = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _l === void 0 ? void 0 : _l.range)) {
        const ranges = (killRange === null || killRange === void 0 ? void 0 : killRange.split(',')) ||
            ((_o = (_m = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _m === void 0 ? void 0 : _m.range) === null || _o === void 0 ? void 0 : _o.map((range) => `${range[0]}-${range[1]}`)) ||
            [];
        for (const range of ranges) {
            const ports = range.split('-').map(Number);
            const startsAt = ports[0];
            const endsAt = ports[1];
            tasks.push(kill_js_1.kill.range(startsAt, endsAt));
        }
    }
    if (killPID || ((_p = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _p === void 0 ? void 0 : _p.pid)) {
        const PIDs = (killPID === null || killPID === void 0 ? void 0 : killPID.split(',').map(Number)) || ((_q = defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.kill) === null || _q === void 0 ? void 0 : _q.pid) || [];
        tasks.push(kill_js_1.kill.pid(PIDs));
    }
    if (hasEnvFile || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.envFile)) {
        const envFilePath = (_r = (0, get_arg_js_1.getArg)('env-file')) !== null && _r !== void 0 ? _r : defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.envFile;
        tasks.push((0, env_js_1.envFile)(envFilePath));
    }
    const options = {
        platform: (0, get_runtime_js_1.platformIsValid)(platform)
            ? platform
            : (0, get_arg_js_1.hasArg)('node')
                ? 'node'
                : (0, get_arg_js_1.hasArg)('bun')
                    ? 'bun'
                    : (0, get_arg_js_1.hasArg)('deno')
                        ? 'deno'
                        : (0, get_runtime_js_1.platformIsValid)(defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.platform)
                            ? defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.platform
                            : undefined,
        filter: typeof filter === 'string' ? new RegExp((0, list_files_js_1.escapeRegExp)(filter)) : filter,
        exclude: typeof exclude === 'string' ? new RegExp((0, list_files_js_1.escapeRegExp)(exclude)) : exclude,
        parallel,
        concurrency,
        quiet,
        debug,
        failFast,
        deno: {
            allow: denoAllow,
            deny: denoDeny,
            cjs: denoCJS,
        },
        noExit: watchMode,
        beforeEach: 'beforeEach' in defaultConfigs ? defaultConfigs.beforeEach : undefined,
        afterEach: 'afterEach' in defaultConfigs ? defaultConfigs.afterEach : undefined,
    };
    if (debug || (defaultConfigs === null || defaultConfigs === void 0 ? void 0 : defaultConfigs.debug)) {
        write_js_1.Write.hr();
        write_js_1.Write.log(`${(0, format_js_1.format)(' Debug Enabled ').bg('brightBlue')}\n`);
        write_js_1.Write.log(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Paths').bold()}`);
        console.table(dirs);
        write_js_1.Write.log('\n');
        write_js_1.Write.log(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Options').bold()}`);
        console.dir(options, { depth: null, colors: true });
    }
    const watchers = new Set();
    const executing = new Set();
    const interval = Number((0, get_arg_js_1.getArg)('watch-interval')) || 1500;
    let isRunning = false;
    const listenStdin = (input) => {
        if (isRunning || executing.size > 0) {
            return;
        }
        if (String(input).trim() === 'rs') {
            for (const watcher of watchers) {
                watcher.stop();
            }
            watchers.clear();
            resultsClear();
            startTests();
        }
    };
    const resultsClear = () => {
        files_js_1.fileResults.success.clear();
        files_js_1.fileResults.fail.clear();
    };
    const startTests = () => {
        if (isRunning || executing.size > 0) {
            return;
        }
        isRunning = true;
        Promise.all(tasks).then(() => {
            (0, poku_js_1.poku)(dirs, options)
                .then(() => {
                if (watchMode) {
                    node_process_1.default.stdin.removeListener('data', listenStdin);
                    node_process_1.default.removeListener('SIGINT', poku_js_1.onSigint);
                    resultsClear();
                    (0, map_tests_js_1.mapTests)('.', dirs, options.filter, options.exclude).then((mappedTests) => {
                        for (const mappedTest of Array.from(mappedTests.keys())) {
                            const currentWatcher = (0, watch_js_1.watch)(mappedTest, (file, event) => {
                                if (event === 'change') {
                                    const filePath = (0, map_tests_js_1.normalizePath)(file);
                                    if (executing.has(filePath)) {
                                        return;
                                    }
                                    executing.add(filePath);
                                    resultsClear();
                                    const tests = mappedTests.get(filePath);
                                    if (!tests) {
                                        return;
                                    }
                                    (0, poku_js_1.poku)(Array.from(tests), options).then(() => {
                                        setTimeout(() => {
                                            executing.delete(filePath);
                                        }, interval);
                                    });
                                }
                            });
                            currentWatcher.then((watcher) => watchers.add(watcher));
                        }
                    });
                    for (const dir of dirs) {
                        const currentWatcher = (0, watch_js_1.watch)(dir, (file, event) => {
                            if (event === 'change') {
                                if (executing.has(file)) {
                                    return;
                                }
                                executing.add(file);
                                resultsClear();
                                (0, poku_js_1.poku)(file, options).then(() => {
                                    setTimeout(() => {
                                        executing.delete(file);
                                    }, interval);
                                });
                            }
                        });
                        currentWatcher.then((watcher) => watchers.add(watcher));
                    }
                    write_js_1.Write.hr();
                    write_js_1.Write.log(`${(0, format_js_1.format)('Watching:').bold()} ${(0, format_js_1.format)(dirs.join(', ')).underline()}`);
                    node_process_1.default.stdin.setEncoding('utf-8');
                    node_process_1.default.stdin.on('data', listenStdin);
                }
            })
                .finally(() => {
                isRunning = false;
            });
        });
    };
    startTests();
})();
